import 'package:contentsquare_platform_interface/contentsquare_platform_interface.dart';
import 'package:flutter/gestures.dart';
import 'gesture_listener.dart';
import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';
import 'dart:math';

/// CSGestureDetector
/// Detects all supported gesture and locate their position on the screen
class CSGestureDetector extends StatelessWidget {
  /// Child widget
  final Widget child;

  /// Callback for the Tap gesture
  final Function(Offset position)? onTap;

  /// Callback for the onLongPress gesture
  final Function(Offset position)? onLongPress;

  /// Callback for the Tap gesture
  final Function(Offset position, CSGestureData data)? onDrag;

  /// Callback for the Tap gesture
  final Function(Offset position, CSGestureData data)? onFlick;

  /// Creates a [CSGestureDetector]
  const CSGestureDetector({
    Key? key,
    required this.child,
    this.onTap,
    this.onLongPress,
    this.onDrag,
    this.onFlick,
  }) : super(key: key);

  void _handleTap(PointerEvent event) {
    onTap?.call(event.position);
  }

  void _handleLongPress(PointerEvent event) {
    onLongPress?.call(event.position);
  }

  void _handleMoveEnd(MoveEvent event) {
    final gestureDirection = _getPrimaryDirection(event.delta);
    if (gestureDirection == null) return;

    final CSGestureData gestureData = CSGestureData(
        distance: event.delta.distance.toInt(),
        direction: gestureDirection,
        velocity:
            max(event.velocity.dx.abs(), event.velocity.dy.abs()).toInt());

    if (gestureData.velocity >= kMinFlingVelocity) {
      onFlick?.call(event.startLocation, gestureData);
    } else {
      onDrag?.call(event.startLocation, gestureData);
    }
  }

  static CSGestureDirection? _getPrimaryDirection(Offset delta) {
    if (delta.dx == 0 && delta.dy == 0) return null;

    if (delta.dx.abs() <= delta.dy.abs()) {
      /// Vertical Gesture
      /// gestureDirection = CSGestureDirection.down OR CSGestureDirection.up
      return delta.dy > 0 ? CSGestureDirection.down : CSGestureDirection.up;
    }

    /// Horizontal Gesture
    /// gestureDirection = CSGestureDirection.right OR CSGestureDirection.left
    return delta.dx > 0 ? CSGestureDirection.right : CSGestureDirection.left;
  }

  @override
  Widget build(BuildContext context) {
    return GestureListener(
      onTap: _handleTap,
      onLongPress: _handleLongPress,
      onMoveEnd: _handleMoveEnd,
      child: child,
    );
  }
}
