import 'package:contentsquare/src/modules/screengraph/screengraph_node.dart';
import 'package:flutter/rendering.dart';

import 'package:flutter/widgets.dart';

/// Screengraph
class Screengraph {

  final ScreengraphNode _rootNode;

  /// Screengraph
  Screengraph(): this._(_makeScreengraphNode(WidgetsBinding.instance!.renderViewElement!, 0, null, null));

  /// Screengraph around this position
  Screengraph.atPosition(Offset position): this._(_makeScreengraphNode(WidgetsBinding.instance!.renderViewElement!, 0, null, position));

  static ScreengraphNode _makeScreengraphNode(Element element, int index, ScreengraphNode? parent, Offset? position) {
    var elementChildren = element.getChildren();
    if (position != null) {
      elementChildren = elementChildren.where((it) {
        final renderObject = it.element.renderObject;
        if (renderObject == null || renderObject is! RenderBox || !renderObject.hasSize) {
          return true;
        }
        final localOffset = renderObject.globalToLocal(position);
        return renderObject.size.contains(localOffset);
      }).toList();
    }
    final children = elementChildren.map((it) => _makeScreengraphNode(it.element, it.index, null, position)).toList();
    return ScreengraphNode(element: element, index: index, parent: parent, children: children);
  }

  Screengraph._(this._rootNode);

  /// Generate a screengraph of the current Element tree and prefix it with the given [rootPath]
  Future<List> toJson(String rootPath, {required double pixelRatio}) async {
    return [await _rootNode.toJson(pixelRatio: pixelRatio, rootPath: rootPath)];
  }

  /// Scan the element tree for candidate elements around the given position
  String pathAtPosition(Offset position) {
    final List<String> targetedElementsPaths = [];
    _exploreSubtreeAroundPosition(
      position,
      _rootNode,
      onCandidateNodeVisited: (ScreengraphNode node) {
        if (targetedElementsPaths
            .where((element) => element.contains(node.path))
            .isEmpty) {
          targetedElementsPaths.add(node.path);
        }
      },
    );
    return targetedElementsPaths.last;
  }

  /// Explore current screengraph given a [position] and call the provided [onCandidateNodeVisited] on each candidate visited
  /// It ignores sections of the tree that `hitTest` return false with the given [position]
  void _exploreSubtreeAroundPosition(Offset position, ScreengraphNode node,
      {required Function(ScreengraphNode node) onCandidateNodeVisited}) {
    final renderObject = node.element.renderObject;
    if (renderObject != null && renderObject is RenderBox) {
      final localOffset = renderObject.globalToLocal(position);
      if (!renderObject.hitTest(BoxHitTestResult(), position: localOffset)) {
        if (node.parent != null) {
          onCandidateNodeVisited(node.parent!);
        }
        return;
      }
    }

    /// Otherwise, checks if the element is a leaf
    bool isLeafElement = node.children.isEmpty;
    if (isLeafElement) {
      onCandidateNodeVisited(node);
    }

    for (final child in node.children) {
      _exploreSubtreeAroundPosition(position, child,
          onCandidateNodeVisited: onCandidateNodeVisited);
    }
  }
}